Library IEEE;

use IEEE.std_logic_1164.all;

use IEEE.std_logic_unsigned.all;

entity rs232_rx is


PORT (

	clk          			: IN STD_LOGIC ;
	reset         			: IN STD_LOGIC ;
	rxd          			: IN STD_LOGIC ;
	baudClkx16     			: IN STD_LOGIC ;
	
	RXDataBuffer    		: OUT STD_LOGIC_VECTOR ( 7 DOWNTO 0);
	RXDataRdy     			: OUT STD_LOGIC;
	stretchRXD    			: OUT STD_LOGIC;
	test					: OUT STD_LOGIC_VECTOR ( 7 DOWNTO 0)

);
							
end rs232_rx;


architecture rtl of rs232_rx is


	SIGNAL RXDataIn			: STD_LOGIC_VECTOR ( 7 DOWNTO 0);
	SIGNAL RXShift			: STD_LOGIC;
	SIGNAL centreSample		: STD_LOGIC;
	SIGNAL RXSampleClk		: STD_LOGIC;
	SIGNAL RXSampleCountEn	: STD_LOGIC;
	SIGNAL RXSampleCounter	: STD_LOGIC_VECTOR ( 3 DOWNTO 0);
	SIGNAL RXSampleNow		: STD_LOGIC;
	SIGNAL RXBitCount		: STD_LOGIC_VECTOR ( 3 DOWNTO 0);
	SIGNAL RXBitCountDone	: STD_LOGIC;
	SIGNAL iRXDataRdy		: STD_LOGIC;

	SIGNAL stretchCount		: STD_LOGIC_VECTOR ( 19 DOWNTO 0);
	SIGNAL stretchDone		: STD_LOGIC;
	SIGNAL stretchEn		: STD_LOGIC;



	TYPE rxStates IS (RXIDLE, RXSTARTBIT0, RXSTARTBIT1, RXSTARTBIT2, 
	RXSAMPLE0,RXSAMPLE1 ,RXSAMPLE2 , RXDONE0, RXDONE1);
	SIGNAL RXNextState     : rxStates;

	TYPE stretchStates IS (STRETCHIDLE,STARTSTRETCH , STOPSTRETCH);
	SIGNAL stretchState     : stretchStates;


BEGIN

-- shift reg for rxd
l_rx_shift :
PROCESS( clk, reset, RXShift)
BEGIN
    IF ( reset ='1') THEN
         RXDataIn <=  X"00";
    ELSIF ( rising_edge (clk)) THEN
         IF ( RXShift ='1' ) THEN
              RXDataIn <= rxd & RXDataIn( 7 DOWNTO 1 );
         END IF;
    END IF;
END PROCESS;




-- divide by 16 OR 8  to derive sample clock at correct time from
-- divide by 8 for the first sample t get into the middle of the bit.
-- then use divide by 16 to stay in the middle of the bit.

l_rx_sample_counter :
PROCESS( clk, baudClkx16 , reset, RXSampleCountEn,centreSample)
BEGIN
    IF ( reset ='1') THEN
         RXSampleCounter <= X"0";
         RXSampleNow <='0';
    ELSIF ( rising_edge (clk)) THEN
         IF ( RXSampleCountEn ='0' ) THEN
              RXSampleCounter <= X"0";
         ELSIF ( baudClkx16 ='1' ) THEN
              RXSampleCounter <= RXSampleCounter + 1;
         END IF;
         IF ( RXSampleCounter ="1111" AND centreSample ='0') THEN
              RXSampleNow <= '1';
         ELSIF ( RXSampleCounter ="1000" AND centreSample ='1') THEN
              RXSampleNow <= '1';
         ELSE
              RXSampleNow <= '0';
         END IF;
    END IF;
END PROCESS;



-- count to 8 for bit count - 8 DATA 
l_rx_bit_counter :
PROCESS( clk, reset, RXSampleCountEn, RXShift )
BEGIN
    IF ( reset ='1') THEN
         RXBitCount <= X"0";
         RXBitCountDone <='0';
    ELSIF ( rising_edge (clk)) THEN
         IF ( RXSampleCountEn ='0' ) THEN
              RXBitCount <= X"0";
         ELSIF ( RXShift ='1' ) THEN
              RXBitCount <= RXBitCount + 1;
         END IF;
         IF ( RXBitCount ="1000" ) THEN
              RXBitCountDone <= '1';
         ELSE
              RXBitCountDone <= '0';
         END IF;
    END IF;
END PROCESS;


-- double buffer the data
l_rx_buf:
PROCESS( clk, reset, RXDataIn, iRXDataRdy)
BEGIN
    IF ( reset ='1') THEN
         RXDataBuffer <= X"00";
		test(5) <='0';
   ELSIF ( rising_edge (clk)) THEN
         IF ( iRXDataRdy ='1' ) THEN
-- grab data ignoring start and stop bits
              RXDataBuffer <= RXDataIn;
			  IF (  RXDataIn = x"AA" ) THEN
				test(5) <='1';
			ELSE 		
				test(5) <='0';
			END IF;
         END IF;
-- pipe the data rdy signal so appears at same time as data
         RXDataRdy <= iRXDataRdy;
    END IF;
END PROCESS;

-- main rx state machine
l_rx_state :
PROCESS (clk,RXNextState,rxd,reset,RXSampleNow,RXBitCountDone)
BEGIN
	IF ( reset ='1') THEN
        RXNextState <= RXIDLE;
		RXShift <='0';
		centreSample <='0';
		iRXDataRdy <='0';
		RXSampleCountEn <='0';
    ELSIF ( rising_edge (clk)) THEN
		CASE RXNextState IS
	-- wait in IDLE until start of the START bit appears -
	         WHEN RXIDLE =>
	              RXShift <='0';
	              centreSample <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='0';
	              IF ( rxd ='0' ) THEN     
	                   RXNextState <= RXSTARTBIT0;
	              END IF;
	-- centre the sample time in the bit period
	         WHEN RXSTARTBIT0 =>
	              RXShift <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='1';
	              centreSample <='1';

	              IF ( RXSampleNow ='1' ) THEN     
	                   RXNextState <= RXSTARTBIT1;
	              END IF;
	                 
	-- ignore start bit and reset sample counter
	         WHEN RXSTARTBIT1 =>
	              RXShift <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='0';
	              centreSample <='0';
	              RXNextState <= RXSTARTBIT2;
	                   
	-- wait for 1 clk
	         WHEN RXSTARTBIT2 =>
	              RXShift <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='0';
	              centreSample <='0';
	              RXNextState <= RXSAMPLE0;
	                   
	-- wait for next sample time
	         WHEN RXSAMPLE0 =>
	              RXShift <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='1';
	              centreSample <='0';
	              IF ( RXSampleNow ='1' ) THEN     
	                   RXNextState <= RXSAMPLE1;
	              END IF;
	-- sample rxd and check if all the bits are in
	         WHEN RXSAMPLE1 =>
	              RXShift <='1';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='1';
	              centreSample <='0';
	              RXNextState <= RXSAMPLE2;
	 -- sample rxd and check if all the bits are in
	         WHEN RXSAMPLE2 =>
	              RXShift <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='1';
	              centreSample <='0';
	              IF ( RXSampleNow ='1' ) THEN     
	                   RXNextState <= RXSAMPLE2;
	              ELSIF ( RXBitCountDone ='1' ) THEN     
	                   RXNextState <= RXDONE0;
	              ELSE
	                   RXNextState <= RXSAMPLE0;
	              END IF;
	-- got all the bits - buffer the data and signal to world
	         WHEN RXDONE0 =>
	              RXShift <='0';
	              iRXDataRdy <='1';
	              RXSampleCountEn <='0';
	              centreSample <='0';
	              RXNextState <= RXDONE1;

	-- wait for bit stream to be high
	         WHEN RXDONE1 =>
	              RXShift <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='0';
	              centreSample <='0';
	              IF ( rxd ='0' ) THEN     
	                   RXNextState <= RXDONE1;
				  ELSE
		              RXNextState <= RXIDLE;
	              END IF;
	             
	         WHEN OTHERS =>
	              RXShift <='0';
	              iRXDataRdy <='0';
	              RXSampleCountEn <='0';
	              centreSample <='0';
	              RXNextState <= RXIDLE;
	    END CASE;
	END IF;
END PROCESS;


-- rx stretch to drive LED

l_rx_stretch :
PROCESS (clk,stretchState,rxd,reset,stretchDone)
BEGIN
	IF ( reset ='1') THEN
        stretchState <= STRETCHIDLE;
		stretchEn <='0';
    ELSIF ( rising_edge (clk)) THEN
		CASE stretchState IS 
			WHEN STRETCHIDLE =>
				stretchEn <='0';
	            IF ( rxd ='0' ) THEN     
	            	stretchState <= STARTSTRETCH;
				END IF;
			WHEN STARTSTRETCH =>
				stretchEn <='1';
	            IF ( stretchDone ='1' ) THEN     
	            	stretchState <= STOPSTRETCH;
				END IF;
			WHEN STOPSTRETCH =>
				stretchEn <='0';
	            IF ( rxd ='1' ) THEN     
	            	stretchState <= STRETCHIDLE;
				END IF;
	         WHEN OTHERS =>
				stretchEn <='0';
	    END CASE;
	END IF;
END PROCESS;

stretchRXD <= NOT stretchEn;


-- 16 bit count  stretch
l_stretch_counter :
PROCESS( clk, reset, stretchEn,baudClkx16  )
BEGIN
    IF ( reset ='1') THEN
         stretchCount <= X"00000";
         stretchDone <='0';
    ELSIF ( rising_edge (clk)) THEN
         IF ( stretchEn ='0'  ) THEN
              stretchCount <= X"00000";
		ELSIF  ( stretchEn ='1' AND baudClkx16 = '1' ) THEN
			stretchCount <= stretchCount + 1;
		END IF;

		IF ( stretchCount = X"FFFFF") THEN 
			stretchDone <='1';
		ELSE	
			stretchDone <='0';
		END IF;
   END IF;
END PROCESS;


-- test stuff
test(0) <= baudClkx16;
--test(1) <= reset;
--test(2) <= clk;
test(7) <= RXShift;
test(2) <= RXSampleCountEn;
test(3) <= RXBitCountDone;
test(6) <= rxd;
test(4) <= iRXDataRdy;

















END rtl;



