-- rs232 tx test bench

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE IEEE.std_logic_unsigned.ALL;


ENTITY tb_rs232_tx IS
end tb_rs232_tx;

ARCHITECTURE rtl OF tb_rs232_tx IS

  CONSTANT PERIOD       : TIME := 10 ns;
 
  CONSTANT STOPTIME : TIME := 10 us;

	SIGNAL clk			: STD_LOGIC:= '0';
	SIGNAL baudclken			: STD_LOGIC:= '0';
	SIGNAL baudclkenx16			: STD_LOGIC:= '0';
	SIGNAL reset				: STD_LOGIC:= '1';
	SIGNAL RS232OUTLatch		: STD_LOGIC_VECTOR( 9 DOWNTO 0);
	SIGNAL txBitCounter			: STD_LOGIC_VECTOR( 3 DOWNTO 0);
	SIGNAL txCount			: STD_LOGIC_VECTOR( 5 DOWNTO 0);
	SIGNAL txBitCounterEn		: STD_LOGIC;
	SIGNAL txBitCountDone		: STD_LOGIC;
	SIGNAL txLoad 				: STD_LOGIC;
	SIGNAL txShift 				: STD_LOGIC;
	SIGNAL TXingNow 			: STD_LOGIC;
	SIGNAL startTest       		: STD_LOGIC                     := '0';
	SIGNAL stopTest        		: STD_LOGIC                     := '0';
--  TX SIGNALS
	SIGNAL txRequest			: STD_LOGIC;  -- internal request to write data to ftdi chip
	SIGNAL cts			: STD_LOGIC := '1' ; -- clear to send from USB device
	SIGNAL dataOut			: STD_LOGIC_VECTOR( 7 DOWNTO 0);
	SIGNAL wrComplete			: STD_LOGIC;	-- internal signal that write to ftdi chip is complete
	SIGNAL txd					: STD_LOGIC;	-- transmit bit stream to USB device
-- RX Signals
	SIGNAL RXDataBuffer 			: STD_LOGIC_VECTOR( 7 DOWNTO 0);
	SIGNAL RXDataRdy   			: STD_LOGIC;

-- error counter
	SIGNAL errorCount			: STD_LOGIC_VECTOR( 7 DOWNTO 0);



COMPONENT rs232_tx
PORT(
	clk						: IN	STD_LOGIC;
	baudClkEn				: IN 	STD_LOGIC;  -- RS232 Baud Rate clk enable.
	reset					: IN	STD_LOGIC;
-- TX SIGNALS
	txRequest				: IN	STD_LOGIC ;  -- internal request to write data to ftdi chip
	cts				: IN	STD_LOGIC  ; -- clear to send from USB device
	dataOut				: IN STD_LOGIC_VECTOR( 7 DOWNTO 0);
	wrComplete				: OUT	STD_LOGIC;	-- internal signal that write to ftdi chip is complete
	txd						: OUT	STD_LOGIC	-- transmit bit stream to USB device
);
  END COMPONENT;
  
COMPONENT rs232_rx
PORT (

clk          : IN STD_LOGIC ;
reset          : IN STD_LOGIC ;
rxd          : IN STD_LOGIC ;
baudClkx16     : IN STD_LOGIC ;

RXDataBuffer     : OUT STD_LOGIC_VECTOR ( 7 DOWNTO 0);
RXDataRdy     : OUT STD_LOGIC

);

  END COMPONENT;
    
  
  
BEGIN
 	clk <= '0' WHEN stopTest = '1'
             ELSE NOT clk AFTER PERIOD/2;
			 
			 
			 
			 

  reset 	<= '0' AFTER 200 ns;
  startTest <= '1' AFTER 250 ns;

  
  l_uut_tx: rs232_tx
  PORT MAP 
  (
 	clk		=> 	clk,
	baudClkEn		=> baudClkEn,
	reset			=> reset,
-- TX SIGNALS
	txRequest		=> txRequest,
	cts				=> cts,
	dataOut			=> dataout,
	wrComplete		=> wrComplete,
	txd				=> txd
	); 

 l_uut_rx: rs232_rx  
PORT MAP 
(
	clk		=> clk,
	reset		=> reset,
	rxd          		=> txd,
	baudClkx16   		=> baudclkenx16,
	RXDataBuffer 		=> RXDataBuffer,
	RXDataRdy   		=> RXDataRdy
);
  
  
  
  
  

  

l_clocks : PROCESS ( clk, reset, 	stopTest)
BEGIN
	IF ( reset ='1' OR stopTest ='1' ) THEN
		baudClkEn <='0';
		txCount <= "000000";
	ELSIF ( rising_edge ( clk )) THEN
		txCount <= txCount + 1;
		IF ( txCount = "111111" ) THEN
			baudClkEn <= '1' ;
		ELSE
			baudClkEn <= '0' ;
		END IF ;
		IF ( txCount( 1 DOWNTO 0)  = "11" ) THEN
			baudclkenx16 <= '1' ;
		ELSE
			baudclkenx16 <= '0' ;
		END IF ;


	END IF ;
END PROCESS;



  l_send : PROCESS
BEGIN
	WAIT UNTIL startTest = '1';
next_byte: 
	WHILE ( stopTest ='0' ) LOOP
		WAIT UNTIL ( rising_edge ( clk));
  		txRequest <= '1';
		WAIT UNTIL ( rising_edge ( clk) );
 	 	txRequest <= '0';
		WAIT UNTIL  wrComplete = '1';
	END LOOP;
END PROCESS;
  	


l_checker :
PROCESS( clk, reset, stopTest, RXDataRdy )
BEGIN
	IF ( reset ='1' OR stopTest ='1' ) THEN
		errorCount <= x"00";
		dataOut <= x"00";
	ELSIF ( rising_edge ( clk )) THEN
		IF ( RXDataRdy ='1' ) THEN
			IF (RXDataBuffer /= dataOut ) THEN
				errorCount <= errorCount +1 ;
			END IF;
			dataOut <= dataOut + 1;
			IF ( dataOut = x"ff" ) THEN
				stopTest <='1';
			END IF;
		END IF;
  	END IF;
 END PROCESS;
  
  
  
  
  

END rtl;









