-- read and write address decoder



LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE IEEE.std_logic_unsigned.ALL;


ENTITY addrdecoder IS

PORT
(
	clk						: IN	STD_LOGIC;
	reset					: IN	STD_LOGIC;
	deviceReset				: IN	STD_LOGIC;
	controlWr				: IN	STD_LOGIC;
	dataIn					: IN	STD_LOGIC_VECTOR( 7 DOWNTO 0); 
	addrIn					: IN	STD_LOGIC_VECTOR( 7 DOWNTO 0); 
	statusReg				: IN	STD_LOGIC_VECTOR( 16 DOWNTO 0);

	controlReg1 			: OUT 	STD_LOGIC_VECTOR(127 DOWNTO 0);
	controlReg2 			: OUT  	STD_LOGIC_VECTOR(127 DOWNTO 0);
	controlReg3 			: OUT  	STD_LOGIC_VECTOR(127 DOWNTO 0);
	status_data				: OUT	STD_LOGIC_VECTOR( 7 DOWNTO 0); 
	leds					: OUT	STD_LOGIC_VECTOR( 3 DOWNTO 0);
	test					: OUT	STD_LOGIC_VECTOR( 7 DOWNTO 0)
);
							
END addrdecoder;

ARCHITECTURE rtl OF addrdecoder IS
	SIGNAL i_test 					:STD_LOGIC_VECTOR( 7 DOWNTO 0);
BEGIN

test <= i_test;

l_wr_decoder : PROCESS ( clk,reset, controlWr,addrIn)
	BEGIN
		IF ( reset ='1' ) THEN
			controlReg1 <= x"00000000000000000000000000000000";
			controlReg2 <= x"00000000000000000000000000000000";
			controlReg3 <= x"00000000000000000000000000000000";
			leds <=x"0";
			i_test <=x"00";
		ELSIF ( rising_edge (clk)) THEN
			-- if running then only control reg and LEDs are accessible
			IF (controlWr ='1') THEN
				CASE ( addrIn ) IS
					WHEN x"00" =>
						controlReg1(7 DOWNTO 0 ) <= dataIn;
						i_test(0) <= NOT i_test(0);
					WHEN x"01" =>
						controlReg1(15 DOWNTO 8 ) <= dataIn;
					WHEN x"02" =>
						controlReg1(23 DOWNTO 16 ) <= dataIn;
					WHEN x"03" =>
						controlReg1(31 DOWNTO 24 ) <= dataIn;
					WHEN x"04" =>
						controlReg1(39 DOWNTO 32 ) <= dataIn;
					WHEN x"05" =>
						controlReg1(47 DOWNTO 40 ) <= dataIn;
					WHEN x"06" =>
						controlReg1(55 DOWNTO 48 ) <= dataIn;
					WHEN x"07" =>
						controlReg1(63 DOWNTO 56 ) <= dataIn;
					WHEN x"08" =>
						controlReg1(71 DOWNTO 64 ) <= dataIn;
					WHEN x"09" =>
						controlReg1(79 DOWNTO 72) <= dataIn;
					WHEN x"0A" =>
						controlReg1(87 DOWNTO 80) <= dataIn;
					WHEN x"0B" =>
						controlReg1(95 DOWNTO 88) <= dataIn;
					WHEN x"0C" =>
						controlReg1(103 DOWNTO 96) <= dataIn;
					WHEN x"0D" =>
						controlReg1(111 DOWNTO 104) <= dataIn;
					WHEN x"0E" =>
						controlReg1(119 DOWNTO 112) <= dataIn;
					WHEN x"0F" =>
						controlReg1(127 DOWNTO 120) <= dataIn;
					WHEN x"10" =>
						controlReg2(7 DOWNTO 0 ) <= dataIn;
					WHEN x"11" =>
						controlReg2(15 DOWNTO 8 ) <= dataIn;
					WHEN x"12" =>
						controlReg2(23 DOWNTO 16 ) <= dataIn;
					WHEN x"13" =>
						controlReg2(31 DOWNTO 24 ) <= dataIn;
					WHEN x"14" =>
						controlReg2(39 DOWNTO 32 ) <= dataIn;
					WHEN x"15" =>
						controlReg2(47 DOWNTO 40 ) <= dataIn;
					WHEN x"16" =>
						controlReg2(55 DOWNTO 48 ) <= dataIn;
					WHEN x"17" =>
						controlReg2(63 DOWNTO 56 ) <= dataIn;
					WHEN x"18" =>
						controlReg2(71 DOWNTO 64 ) <= dataIn;
					WHEN x"19" =>
						controlReg2(79 DOWNTO 72) <= dataIn;
					WHEN x"1A" =>
						controlReg2(87 DOWNTO 80) <= dataIn;
					WHEN x"1B" =>
						controlReg2(95 DOWNTO 88) <= dataIn;
					WHEN x"1C" =>
						controlReg2(103 DOWNTO 96) <= dataIn;
					WHEN x"1D" =>
						controlReg2(111 DOWNTO 104) <= dataIn;
					WHEN x"1E" =>
						controlReg2(119 DOWNTO 112) <= dataIn;
					WHEN x"1F" =>
						controlReg2(127 DOWNTO 120) <= dataIn;
					WHEN x"20" =>
						controlReg3(7 DOWNTO 0 ) <= dataIn;
					WHEN x"21" =>
						controlReg3(15 DOWNTO 8 ) <= dataIn;
					WHEN x"22" =>
						controlReg3(23 DOWNTO 16 ) <= dataIn;
					WHEN x"23" =>
						controlReg3(31 DOWNTO 24 ) <= dataIn;
					WHEN x"24" =>
						controlReg3(39 DOWNTO 32 ) <= dataIn;
					WHEN x"25" =>
						controlReg3(47 DOWNTO 40 ) <= dataIn;
					WHEN x"26" =>
						controlReg3(55 DOWNTO 48 ) <= dataIn;
					WHEN x"27" =>
						controlReg3(63 DOWNTO 56 ) <= dataIn;
					WHEN x"28" =>
						controlReg3(71 DOWNTO 64 ) <= dataIn;
					WHEN x"29" =>
						controlReg3(79 DOWNTO 72) <= dataIn;
					WHEN x"2A" =>
						controlReg3(87 DOWNTO 80) <= dataIn;
					WHEN x"2B" =>
						controlReg3(95 DOWNTO 88) <= dataIn;
					WHEN x"2C" =>
						controlReg3(103 DOWNTO 96) <= dataIn;
					WHEN x"2D" =>
						controlReg3(111 DOWNTO 104) <= dataIn;
					WHEN x"2E" =>
						controlReg3(119 DOWNTO 112) <= dataIn;
					WHEN x"2F" =>
						controlReg3(127 DOWNTO 120) <= dataIn;
					WHEN x"30" =>
						leds <= dataIn( 3 DOWNTO 0);
					WHEN OTHERS =>
				END CASE;
			END IF ;
		END IF;
	END PROCESS;	


l_rd_decoder : PROCESS ( clk,reset, statusReg,addrIn)
	BEGIN
		IF ( reset ='1' ) THEN
			status_data <= statusReg(7 DOWNTO 0);
		ELSIF ( rising_edge (clk)) THEN
			CASE ( addrIn ) IS
				WHEN x"00" =>
					status_data <= statusReg(7 DOWNTO 0);
				WHEN x"01" =>
					status_data <= statusReg(15 DOWNTO 8);
				WHEN x"02" =>
					status_data <= "0000000" & statusReg(16);
				WHEN OTHERS =>
					status_data <= "0000000" & statusReg(16);
			END CASE;
		END IF;
	END PROCESS;	
END rtl;

