-- clock generation
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY clocks IS

PORT
(
	reset					: IN	STD_LOGIC;
	clkin					: IN	STD_LOGIC; -- will be 48MHz
	SlowSysClk				: IN	STD_LOGIC; -- will be 12MHz
	baudClkx16En			: OUT 	STD_LOGIC; -- is slowSysClk/4 - 3MHz
	baudClkEn				: OUT 	STD_LOGIC; -- is slowSysClk/64 - 187.5kHz
	fastSysClk				: OUT	STD_LOGIC; -- currently 48M - will become 96 if get pll going
	slowClkOut				: OUT	STD_LOGIC  -- to be 12MHz
);
							
END clocks;

ARCHITECTURE rtl OF clocks IS

COMPONENT pll IS
	PORT
	(
		inclock		: IN STD_LOGIC ;
		clock0		: OUT STD_LOGIC ;
		clock1		: OUT STD_LOGIC 
	);
END COMPONENT;
	SIGNAL	iFastSysClk			: STD_LOGIC;
	SIGNAL	iusbClk				: STD_LOGIC_VECTOR( 4 DOWNTO 0 );
	SIGNAL	ibaudCounter		: STD_LOGIC_VECTOR( 5 DOWNTO 0 );
	SIGNAL	locked				: STD_LOGIC;
	SIGNAL	iUSBClkx2Out		: STD_LOGIC;
	SIGNAL	slowSysClockCounter	: STD_LOGIC_VECTOR( 2 DOWNTO 0 );


BEGIN

 iFastSysClk <= clkin;
 FastSysClk <= iFastSysClk;

-- generate slow clock for use in the PC interface
PROCESS ( iFastSysClk,reset )
BEGIN
	IF ( reset ='1'  ) THEN
		slowSysClockCounter <= "000";
	ELSIF ( risIng_edge( iFastSysClk ) ) THEN
		slowSysClockCounter <= slowSysClockCounter + 1;
	END IF;
END PROCESS;

slowClkOut <= slowSysClockCounter(1);  -- if 96M then use (2);




-- baud rate generator from slowsysclk - generates both 16x clk and a baud rate clock
PROCESS ( slowSysClk,ibaudCounter,reset )
BEGIN
	IF ( reset ='1'  ) THEN
		ibaudCounter <= "000000";
		baudClkx16En <= '0';
		baudClkEn <= '0';
	ELSIF ( risIng_edge( slowSysClk ) ) THEN

		ibaudCounter <= ibaudCounter + 1 ;
		IF ( ibaudCounter( 1 DOWNTO 0 ) = "11" ) THEN
			baudClkx16En <='1';
		ELSE
			baudClkx16En <='0';
		END IF;
		IF ( ibaudCounter = "111111" ) THEN
			baudClkEn <='1';
		ELSE
			baudClkEn <='0';
		END IF;
	END IF;
END PROCESS;





END rtl;










