

-- contains the pulse generator logic
-- Modes are : 
-- mode(0) = gate enable
-- mode(1) = gate invert
-- mode(2) = output invert
-- mode(3) = astable/mono
-- mode(4) = retrig
-- mode(5) = linked
-- mode(6) = jitter enable
-- mode(7) = burst enable
-- mode(8) = 
-- mode(9) = 
-- mode(10) =
-- mode(11) =
-- mode(12) =
-- mode(13) =
-- mode(14) = manual trigger
-- mode(15) = output enable

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY pulse IS
 
	PORT
	(	
		clk	  				: IN  STD_LOGIC;
		reset				: IN  STD_LOGIC;
		chan_num			: IN  STD_LOGIC;
		gate 				: IN  STD_LOGIC;	
		manual_gate			: IN  STD_LOGIC;	
	 	done_skew			: IN  STD_LOGIC;
		sync_in				: IN  STD_LOGIC;	
		control_wr_complete	: IN  STD_LOGIC;
		jitter_clk_enable	: IN  STD_LOGIC;
 		mode				: IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
		jitter				: IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
		burst				: IN  STD_LOGIC_VECTOR( 7 DOWNTO 0);
		on_count			: IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
		off_count			: IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
	 	enable_skew			: OUT STD_LOGIC;
		real_pulse			: OUT STD_LOGIC;		
		sync_out			: OUT STD_LOGIC;	
		linked				: OUT STD_LOGIC;	
		test				: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
							
END pulse;

ARCHITECTURE rtl OF pulse IS
------------------------
-- Declare COMPONENTS --
------------------------
COMPONENT downcounter IS
	GENERIC
	(
		size			: NATURAL := 7 
	);
	PORT
	(	
		clk	  			: IN  STD_LOGIC;
		clken	  		: IN  STD_LOGIC;
		reset			: IN  STD_LOGIC;
		enable			: IN  STD_LOGIC;
		count			: IN  STD_LOGIC_VECTOR(size -1 DOWNTO 0);
		done			: OUT STD_LOGIC
	);

END COMPONENT;
COMPONENT edge_detector IS

	PORT
	(	
		clk				: IN  STD_LOGIC;
		input			: IN  STD_LOGIC;
		reset			: IN  STD_LOGIC;
		rising			: OUT  STD_LOGIC;
		falling			: OUT  STD_LOGIC
	);

END COMPONENT;

----------------------
-- Declare Signals  --
----------------------
 	SIGNAL enable_on			: STD_LOGIC;
 	SIGNAL enable_off			: STD_LOGIC;
 	SIGNAL enable_jitter		: STD_LOGIC;
	SIGNAL enable_jitter_now	: STD_LOGIC;
 	SIGNAL enable_burst			: STD_LOGIC;
	SIGNAL enable_burst_clk		: STD_LOGIC;
 	SIGNAL done_on				: STD_LOGIC;
 	SIGNAL done_off				: STD_LOGIC;
 	SIGNAL done_burst			: STD_LOGIC;
 	SIGNAL done_jitter			: STD_LOGIC;
 	SIGNAL done_jitter_now		: STD_LOGIC;
	SIGNAL i_pulse				: STD_LOGIC;
	SIGNAL pulse				: STD_LOGIC;
 	SIGNAL trigger_rising		: STD_LOGIC;
 	SIGNAL trigger				: STD_LOGIC;
 	SIGNAL manual_trigger_rising: STD_LOGIC;
 	SIGNAL trigger_falling		: STD_LOGIC;
 	SIGNAL manual_trigger_falling	: STD_LOGIC;
	SIGNAL jitter_now			: STD_LOGIC_VECTOR(15 DOWNTO 0);
	SIGNAL	itest				: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	jitter_count 		: STD_LOGIC_VECTOR(2 DOWNTO 0);
	SIGNAL	jitter_enable 		: STD_LOGIC;
	SIGNAL	m_gate_enable		: STD_LOGIC;
	SIGNAL	m_gate_invert       : STD_LOGIC;	
	SIGNAL	m_output_invert     : STD_LOGIC;	
	SIGNAL	m_monostable	    : STD_LOGIC;	
	SIGNAL	m_retrig           	: STD_LOGIC; 	
	SIGNAL	m_linked           	: STD_LOGIC;		
	SIGNAL	m_jitter_enable    	: STD_LOGIC; 	
	SIGNAL	m_burst_enable     	: STD_LOGIC; 	
	SIGNAL	m_manual_trigger   	: STD_LOGIC; 	
	SIGNAL	m_output_enable    	: STD_LOGIC; 	
	
	SIGNAL	basic_astable    	: STD_LOGIC; 
	SIGNAL	basic_monostable   	: STD_LOGIC; 
	SIGNAL	burst_astable   	: STD_LOGIC; 
	SIGNAL	burst_monostable   	: STD_LOGIC; 
	SIGNAL	linked_monostable  	: STD_LOGIC; 
	SIGNAL	linked_burst_mono  	: STD_LOGIC; 
 	SIGNAL  sync_in_rising		: STD_LOGIC;
 	SIGNAL  sync_in_falling		: STD_LOGIC;
 	SIGNAL  i_enable_skew		: STD_LOGIC;
	SIGNAL	iburst				: STD_LOGIC_VECTOR(7 DOWNTO 0);
	
	
----------------------------------
-- Declare state machine types  --
----------------------------------

	TYPE pulse_states IS 
	(
		idle,
		s_mode0_0, s_mode0_1, 						-- mode 0 basic astable
		s_mode0_2, s_mode0_3, s_mode0_4,			
		s_mode1_0, s_mode1_1, 						-- mode 1 basic monostable
		s_mode1_2, s_mode1_3, s_mode1_4, 	
		s_mode1_5, s_mode1_6, 
		s_mode2_0, s_mode2_1,				 		-- mode 2 astable with burst
		s_mode2_2, s_mode2_3, s_mode2_4, 						
		s_mode2_5, s_mode2_6,		
		s_mode3_0, s_mode3_1,				 		-- mode 3 monostable with burst
		s_mode3_2, s_mode3_3, s_mode3_4, 						
		s_mode3_5, s_mode3_6, 		
		s_mode4_0, s_mode4_1,				 		-- mode 4 linked monostable
		s_mode4_2, s_mode4_3, s_mode4_4, 						
		s_mode4_5, s_mode4_6, s_mode4_7,		
		s_mode5_0, s_mode5_1,				 		-- mode 5 monostable with burst
		s_mode5_2, s_mode5_3, s_mode5_4, 						
		s_mode5_5, s_mode5_6, s_mode5_7,		
		
		both_off
	);
	SIGNAL pulse_state		: pulse_states;



----------------------
-- Code Starts Here --
----------------------

BEGIN
-- horrible kludge

iBurst <= burst + 1 when m_monostable ='1'
	    else burst;

--------------------------------------------
-- convert mode input to independant control signals
--------------------------------------------
m_gate_enable		<=	mode(0) ;
m_gate_invert       <=  mode(1) ; 
m_output_invert     <=  mode(2) ;
m_monostable		<=  mode(3) ; 
m_retrig            <=  mode(4) ;
m_linked           	<=  mode(5) ;
m_jitter_enable     <=  mode(6) ;
m_burst_enable      <=  mode(7) ;
m_manual_trigger    <=  mode(14);
m_output_enable     <=  mode(15);
                      -- mode(8) ;
                      -- mode(9) ;
                      -- mode(10);
                      -- mode(11);
                      -- mode(12);
                      -- mode(13);                     
linked <= m_linked;


enable_skew <= i_enable_skew;
--------------------------------------------
-- instantiate edge detector for Triggers	  --
--------------------------------------------
trigger <= gate WHEN m_gate_invert = '0'
				else NOT gate;

l_trigger : edge_detector
	PORT MAP
	(	
		clk	=> clk,
		input	=> trigger,
		reset	=> reset,
		rising	=> trigger_rising,
		falling	=> trigger_falling
	);


	l_manual_trigger : edge_detector
	PORT MAP
	(	
		clk	=> clk,
		input	=> m_manual_trigger,
		reset	=> reset,
		rising	=> manual_trigger_rising,
		falling	=> manual_trigger_falling
	);
	l_sync_in_trigger : edge_detector
	PORT MAP
	(	
		clk	=> clk,
		input	=> sync_in,
		reset	=> reset,
		rising	=> sync_in_rising,
		falling	=> sync_in_falling
	);



--------------------------------------------
-- instantiate on & off counters	  --
--------------------------------------------
l_on : downcounter
	GENERIC MAP 
	(
		size => 32
	)
	PORT MAP
	(	
		clk		=> clk,
		clken	=> '1',
		reset	=> reset,
		enable	=> enable_on,
		count	=> on_count,
		done	=> done_on
	);
l_off : downcounter
	GENERIC MAP 
	(
		size => 32
	)
	PORT MAP
	(	
		clk		=> clk,
		clken	=> '1',
		reset	=> reset,
		enable	=> enable_off,
		count	=> off_count,
		done	=> done_off
	);

l_burst : downcounter
	GENERIC MAP 
	(
		size => 8
	)
	PORT MAP
	(	
		clk		=> clk,
		clken	=> enable_burst_clk,
		reset	=> reset,
		enable	=> enable_burst,
		count	=> iBurst,
		done	=> done_burst
	);


l_jitter : downcounter
	GENERIC MAP 
	(
		size => 16
	)
	PORT MAP
	(	
		clk	=> clk,
		clken	=> '1',
		reset	=> reset,
		enable	=> enable_jitter,
		count	=> jitter_now,
		done	=> done_jitter
	);



l_jitter_proc:PROCESS( reset,clk, jitter, done_jitter_now, jitter_clk_enable)
BEGIN
	IF ( reset ='1'  ) THEN
		jitter_now   <= jitter;
		itest(2) <='0';
	ELSIF ( rising_edge(clk)) THEN
		-- vary the amount of jitter from 0 to jitter	
		IF ( jitter_clk_enable = '1' ) THEN
			IF ( jitter_now = x"0000" ) THEN
				jitter_now <= jitter;
			ELSE
				jitter_now <= jitter_now - 1;
			END IF;
		
		END IF;
	END IF;		
END PROCESS;

-- combinatorial control terms for statemachine
basic_astable <= '1' WHEN   m_monostable = '0' AND m_burst_enable ='0' AND 
							( chan_num = '0' OR ( chan_num ='1' AND m_linked ='0') )
					ELSE '0';
basic_monostable <= '1' WHEN   m_monostable ='1' AND m_burst_enable ='0' AND 
							( chan_num = '0' OR ( chan_num ='1' AND m_linked ='0' ) )
					ELSE '0';
burst_astable <= '1' WHEN   m_monostable ='0' AND m_burst_enable ='1' AND 
							( chan_num = '0' OR ( chan_num ='1' AND m_linked ='0' ) )
					ELSE '0';
burst_monostable <= '1' WHEN   m_monostable ='1' AND m_burst_enable ='1' AND 
							( chan_num = '0' OR ( chan_num ='1' AND m_linked ='0' ) )
					ELSE '0';
					
linked_monostable <= '1' WHEN chan_num = '1' AND m_burst_enable ='0' AND m_linked ='1' 
					ELSE '0';

linked_burst_mono <= '1' WHEN  chan_num = '1' AND m_burst_enable ='1' AND m_linked ='1' 
					ELSE '0';
-----------------------------------
-- controlling state machine	 --
-----------------------------------

l_pulse:PROCESS( reset,clk, control_wr_complete,mode,done_burst,done_jitter,
				 done_on,done_off,trigger_rising,manual_trigger_rising)
BEGIN
	IF ( reset ='1' ) THEN
		enable_on			<='0';
		enable_off			<='0';
		enable_burst		<='0';
		enable_burst_clk	<='0';
		pulse_state			<= idle;
		enable_jitter   	<= '0';
		i_pulse				<= '0';
		i_enable_skew			<= '0';
		itest(5) 			<= '0';
		sync_out			<= '0';
	ELSIF ( rising_edge(clk)) THEN
		CASE pulse_state IS
			WHEN idle =>
				enable_on		<='0';
				enable_off		<='0';
				enable_burst	<='0';
				enable_burst_clk	<='0';
				i_enable_skew <= '0';
				i_pulse			<='0';
				enable_jitter   <='0';
				i_enable_skew		<='0';

				sync_out		<='0';
				IF ( m_output_enable ='1' ) THEN 			-- Check if OFF
						pulse_state	<= both_off;
				ELSIF (basic_astable = '1' ) THEN	-- check if basic Astable
					pulse_state	<= s_mode0_0;
				ELSIF ( basic_monostable = '1'  ) THEN	-- check if basic Monostable
					pulse_state	<= s_mode1_0;			
				ELSIF ( burst_astable ='1'  ) THEN	-- check if Astable with burst
					pulse_state	<= s_mode2_0;
				ELSIF ( burst_monostable = '1'  )THEN	-- check if Monostable with burst
					pulse_state	<= s_mode3_0;
				ELSIF ( linked_monostable = '1'  )THEN		-- check if linked Monostable
					pulse_state	<= s_mode4_0;
				ELSIF ( linked_burst_mono = '1'  )THEN	-- check if linked Monostable with burst
					pulse_state	<= s_mode5_0;
				END IF ;
-- basic periodic clock
-- with gated output
	
			WHEN s_mode0_0 =>
				i_pulse <='0';
				enable_off <='1';
				enable_on <='0';
				enable_jitter <='0';
				i_enable_skew <='0';											-- enable skew if requird
				sync_out		<='0';
				IF ( basic_astable ='0' ) THEN		-- check if basic Astable
					pulse_state <= idle;
				ELSIF  ( done_off ='1' AND m_jitter_enable ='0' ) THEN
					pulse_state <= s_mode0_2;
				ELSIF ( done_off ='1') THEN
					pulse_state <= s_mode0_1;
				END IF;	
			-- jitter rising edge
			WHEN s_mode0_1 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='1';
				IF ( basic_astable ='0') THEN		-- check if Astable
					pulse_state <= idle;
				ELSIF  ( done_jitter = '1' ) THEN
					pulse_state <= s_mode0_2;
				END IF;	
			-- start 'on' down counter and wait until complete
			WHEN s_mode0_2 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				sync_out	<='1';
				IF ( basic_astable ='0' ) THEN		-- check if Astable
					pulse_state <= idle;
				ELSIF  ( done_on ='1' AND m_jitter_enable ='0' ) THEN
					pulse_state <= s_mode0_4;
				ELSIF ( done_on ='1') THEN
					pulse_state <= s_mode0_3;
				END IF;	
			-- jitter falling edge
			WHEN s_mode0_3 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='1';
				IF ( basic_astable ='0' ) THEN		-- check if Astable
					pulse_state <= idle;
				ELSIF  ( done_jitter = '1' ) THEN
						pulse_state <= s_mode0_4;
				END IF;
			-- decide what to do next
			WHEN s_mode0_4 =>
				i_enable_skew <='0';					-- enable skew if requird
				IF ( basic_astable ='0' ) THEN		-- check if Astable
					pulse_state <= idle;
				ELSE				
					pulse_state <= s_mode0_0;				
				END IF;
				
-- basic monstable
			--wait for rising edge on trigger
			WHEN s_mode1_0 =>
				enable_burst_clk	<='0';
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				i_enable_skew <='0';											-- enable skew if requird
				sync_out		<='0';
				IF ( basic_monostable = '0'  )	THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF  ( ( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_jitter_enable ='1' ) THEN   -- check for trigger 
					pulse_state <= s_mode1_1;
				ELSIF (( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_jitter_enable ='0' ) THEN	-- check for trigger
					pulse_state <= s_mode1_2;
				END IF;	
					-- jitter rising edge
			WHEN s_mode1_1 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='1';
				i_enable_skew <='0';											-- enable skew if requird
				IF (   basic_monostable = '0'  ) THEN		-- check if  basic monostable
					pulse_state <= idle;
				ELSIF  ( done_jitter = '1' ) THEN
					pulse_state <= s_mode1_2;
				END IF;	
			-- start 'on' down counter and wait until complete
			WHEN s_mode1_2 =>
				itest(5) <= NOT itest(5);
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				sync_out	<='1';
				i_enable_skew <='0';											-- enable skew if requird
				IF (  basic_monostable = '0'  ) THEN		-- check if basic monostable
					pulse_state <= idle;
				ELSIF ( ( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_retrig = '1' ) THEN		-- CHECK IF RETRIGGERED
					pulse_state <= s_mode1_5;
				ELSIF  ( done_on ='1' AND m_jitter_enable ='0' ) THEN
					pulse_state <= s_mode1_4;						-- decide what to do next
				ELSIF ( done_on ='1' ) THEN
					pulse_state <= s_mode1_3;						-- jitter edge
				END IF;	
			-- jitter falling edge
			WHEN s_mode1_3 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='1';
				IF (   basic_monostable = '0' ) THEN		-- check if monostable
					pulse_state <= idle;
				ELSIF ( ( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_retrig = '1' ) THEN		-- CHECK IF RETRIGGERED
					pulse_state <= s_mode1_5;
				ELSIF  ( done_jitter = '1' ) THEN
					pulse_state <= s_mode1_4;						-- decide what to do next
				END IF;			
			-- decide what to do next
			WHEN s_mode1_4 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				sync_out		<='0';
				IF (   basic_monostable = '0' ) THEN		-- check if monostable
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode1_0;				
				END IF;
			-- re trigger event - so restart down count
			WHEN s_mode1_5 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				sync_out		<='1';
				pulse_state <= s_mode1_6;				
			-- re trigger event - so restart down count
			WHEN s_mode1_6 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				sync_out		<='1';
				enable_jitter <='0';
				pulse_state <= s_mode1_2;				

-- astable with burst    
			-- start 'off' down counter  
					
			WHEN s_mode2_0 =>
				i_pulse <='0';
				enable_off <='1';
				enable_on <='0';
				enable_burst <= '0';
				enable_burst_clk <= '0';
				enable_jitter <= '0';
				sync_out		<='0';
				IF ( burst_astable ='0') THEN		-- check if burst Astable
					pulse_state <= idle;
				ELSIF ( done_off ='1') THEN
					pulse_state <= s_mode2_1;
				END IF;	
			-- start 'on' down counter and wait until complete, load up burst counter
			WHEN s_mode2_1 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF ( burst_astable ='0') THEN		-- check if Astable
					pulse_state <= idle;
				ELSIF ( done_on ='1') THEN
					pulse_state <= s_mode2_2;
				END IF;	
			-- using 'on'  down counter for the pulse timing - so needs reseting
			-- incrememnt burst counter
				WHEN s_mode2_2 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '1';
				sync_out		<='1';
				IF (burst_astable ='0') THEN		-- check if Astable
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode2_3;
				END IF;	

				WHEN s_mode2_3 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF ( burst_astable ='0') THEN		-- check if Astable
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode2_4;								-- burst not done, so pulse again
				END IF;	

				-- reset 'on' down counter for the pulse off time
				WHEN s_mode2_4 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF ( burst_astable ='0' ) THEN		-- check if Astable
					pulse_state <= idle;
				ELSIF ( done_on ='1' AND done_burst ='1'  ) THEN
					pulse_state <= s_mode2_0;								-- burst done so do off period
				ELSIF ( done_on ='1' ) THEN
					pulse_state <= s_mode2_5;								-- burst not done, so pulse again
				END IF;	

				-- reset 'on' down counter for the pulse off time
			
			-- using 'on'  down counter for the pulse timing - so needs reseting
				WHEN s_mode2_5 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				IF ( burst_astable ='0') THEN		-- check if Astable
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode2_6;
				END IF;	
			
			-- pause to reset counter
				WHEN s_mode2_6 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				IF ( burst_astable ='0') THEN		-- check if Astable
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode2_1;
				END IF;	
			
				
	
-- monostable with burst  - 

			--wait for rising edge on trigger

			WHEN s_mode3_0 =>
				i_pulse <='0';
				enable_off <='1';
				enable_on <='0';
				enable_burst <= '0';
				enable_burst_clk <= '0';
				enable_jitter <= '0';
				sync_out		<='0';
				i_enable_skew <= '0';
				IF ( burst_monostable ='0'  )	THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF  ( ( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_jitter_enable ='1' ) THEN
					pulse_state <= s_mode3_1;
				ELSIF (( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_jitter_enable ='0' ) THEN
					pulse_state <= s_mode3_2;
				END IF;	
			-- start 'on' down counter and wait until complete, load up burst counter
			WHEN s_mode3_1 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				sync_out		<='1';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				i_enable_skew <= '0';
				IF (  burst_monostable ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF ( done_on ='1') THEN
					pulse_state <= s_mode3_2;
				END IF;	
			-- using 'on'  down counter for the pulse timing - so needs reseting
			-- incrememnt burst counter
				WHEN s_mode3_2 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '1';
				sync_out		<='1';
				i_enable_skew <= '0';
				IF (  burst_monostable ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode3_3;
				END IF;	

				WHEN s_mode3_3 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  burst_monostable ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode3_4;								-- burst not done, so pulse again
				END IF;	

				-- reset 'on' down counter for the pulse off time
				WHEN s_mode3_4 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  burst_monostable ='0'  ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF ( done_on ='1' AND done_burst ='1'  ) THEN
					pulse_state <= s_mode3_0;								-- burst done so do off period
				ELSIF ( done_on ='1' ) THEN
					pulse_state <= s_mode3_5;								-- burst not done, so pulse again
				END IF;	

				-- reset 'on' down counter for the pulse off time
			
			-- using 'on'  down counter for the pulse timing - so needs reseting
				WHEN s_mode3_5 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  burst_monostable ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode3_6;
				END IF;	
			
			-- pause to reset counter
				WHEN s_mode3_6 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  burst_monostable ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode3_1;
				END IF;	

-- linked  monstable
			--wait for rising edge on trigger
			WHEN s_mode4_0 =>
				enable_burst_clk	<='0';
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				i_enable_skew <='0';											-- enable skew if requird
				sync_out		<='0';
				IF ( linked_monostable = '0'  )	THEN	-- check still in monostable mode
					pulse_state <= idle;
				ELSIF ( sync_in_rising ='1'  ) THEN		-- check for trigger
					pulse_state <= s_mode4_7;
				END IF;	
				-- start skew counter 
			WHEN s_mode4_7 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				i_enable_skew <='1';							-- enable skew
				IF (   linked_monostable = '0'  ) THEN		-- check if  basic monostable
					pulse_state <= idle;
				ELSIF  ( done_skew = '1'  AND m_jitter_enable ='1' ) THEN   -- check for skew done 
					pulse_state <= s_mode4_1;
				ELSIF (done_skew = '1'  AND m_jitter_enable ='0' ) THEN		-- check for skew done
					pulse_state <= s_mode4_2;
				END IF;	

					-- jitter rising edge
			WHEN s_mode4_1 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='1';
				i_enable_skew <='0';											-- enable skew if requird
				IF (   linked_monostable = '0'  ) THEN		-- check if  basic monostable
					pulse_state <= idle;
				ELSIF  ( done_jitter = '1' ) THEN
					pulse_state <= s_mode4_2;
				END IF;	
			-- start 'on' down counter and wait until complete
			WHEN s_mode4_2 =>
				itest(5) <= NOT itest(5);
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				sync_out	<='1';
				i_enable_skew <='0';											-- enable skew if requird
				IF (  linked_monostable = '0'  ) THEN		-- check if basic monostable
					pulse_state <= idle;
				ELSIF ( ( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_retrig = '1' ) THEN		-- CHECK IF RETRIGGERED
					pulse_state <= s_mode4_5;
				ELSIF  ( done_on ='1' AND m_jitter_enable ='0' ) THEN
					pulse_state <= s_mode4_4;						-- decide what to do next
				ELSIF ( done_on ='1' ) THEN
					pulse_state <= s_mode4_3;						-- jitter edge
				END IF;	
			-- jitter falling edge
			WHEN s_mode4_3 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='1';
				IF (   linked_monostable = '0' ) THEN		-- check if monostable
					pulse_state <= idle;
				ELSIF ( ( trigger_rising ='1' OR manual_trigger_rising ='1' ) AND m_retrig = '1' ) THEN		-- CHECK IF RETRIGGERED
					pulse_state <= s_mode4_5;
				ELSIF  ( done_jitter = '1' ) THEN
					pulse_state <= s_mode4_4;						-- decide what to do next
				END IF;			
			-- decide what to do next
			WHEN s_mode4_4 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				sync_out		<='0';
				IF (   linked_monostable = '0' ) THEN		-- check if monostable
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode4_0;				
				END IF;
			-- re trigger event - so restart down count
			WHEN s_mode4_5 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				sync_out		<='1';
				pulse_state <= s_mode4_6;				
			-- re trigger event - so restart down count
			WHEN s_mode4_6 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				sync_out		<='1';
				enable_jitter <='0';
				pulse_state <= s_mode4_2;				


-- linked monostable with burst  - 

			--wait for rising edge on trigger

			WHEN s_mode5_0 =>
				i_pulse <='0';
				enable_off <='1';
				enable_on <='0';
				enable_burst <= '0';
				enable_burst_clk <= '0';
				enable_jitter <= '0';
				sync_out		<='0';
				i_enable_skew <= '0';
				IF ( linked_burst_mono ='0'  )	THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF ( sync_in_rising ='1'  ) THEN		-- check for trigger					
					pulse_state <= s_mode5_7;
				END IF;

			WHEN s_mode5_7 =>
				i_pulse <='0';
				enable_off <='1';
				enable_on <='0';
				enable_burst <= '0';
				enable_burst_clk <= '0';
				enable_jitter <= '0';
				sync_out		<='0';
				i_enable_skew <= '1';
				IF ( linked_burst_mono ='0'  )	THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF  ( done_skew ='1'  AND m_jitter_enable ='1' ) THEN -- check for skew done
					pulse_state <= s_mode5_1;
				ELSIF ( done_skew ='1'  AND m_jitter_enable ='0' ) THEN  -- check for skew done
					pulse_state <= s_mode5_2;
				END IF;	

			-- start 'on' down counter and wait until complete, load up burst counter
			WHEN s_mode5_1 =>
				i_pulse <='1';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				sync_out		<='1';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				i_enable_skew <= '0';
				IF (  linked_burst_mono ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF ( done_on ='1') THEN
					pulse_state <= s_mode5_2;
				END IF;	
			-- using 'on'  down counter for the pulse timing - so needs reseting
			-- incrememnt burst counter
				WHEN s_mode5_2 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '1';
				sync_out		<='1';
				i_enable_skew <= '0';
				IF (  linked_burst_mono ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode5_3;
				END IF;	

				WHEN s_mode5_3 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  linked_burst_mono ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode5_4;								-- burst not done, so pulse again
				END IF;	

				-- reset 'on' down counter for the pulse off time
				WHEN s_mode5_4 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='1';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  linked_burst_mono ='0'  ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSIF ( done_on ='1' AND done_burst ='1'  ) THEN
					pulse_state <= s_mode5_0;								-- burst done so do off period
				ELSIF ( done_on ='1' ) THEN
					pulse_state <= s_mode5_5;								-- burst not done, so pulse again
				END IF;	

				-- reset 'on' down counter for the pulse off time
			
			-- using 'on'  down counter for the pulse timing - so needs reseting
				WHEN s_mode5_5 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  linked_burst_mono ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode5_6;
				END IF;	
			
			-- pause to reset counter
				WHEN s_mode5_6 =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				enable_burst <= '1';
				enable_burst_clk <= '0';
				sync_out		<='1';
				IF (  linked_burst_mono ='0' ) THEN		-- check still in monostable mode
					pulse_state <= idle;
				ELSE
					pulse_state <= s_mode5_1;
				END IF;	

			-- turn off output
			WHEN both_off =>
				i_pulse <='0';
				enable_off <='0';
				enable_on <='0';
				enable_jitter <='0';
				-- stay put unless the operating mode changes
				IF ( m_output_enable = '1' ) THEN
					pulse_state <= both_off;
				ELSE
					pulse_state <= idle;
				END IF;

			WHEN OTHERS =>
				enable_on	<='0';
				enable_off	<='0';
				enable_burst	<='0';
				pulse_state	<=idle;

		
		END CASE;
	END IF;
END PROCESS;



pulse <= i_pulse WHEN 	m_output_enable = '0' AND m_monostable ='0' AND m_gate_enable ='0'								-- astable with no gate
	ELSE i_pulse WHEN 	m_output_enable = '0' AND m_monostable ='0' AND m_gate_enable ='1' AND m_gate_invert ='0' AND gate ='1'	-- astable with  non inverted gate
	ELSE '0'     WHEN 	m_output_enable = '0' AND m_monostable ='0' AND m_gate_enable ='1' AND m_gate_invert ='0' AND gate ='0'	-- astable with  non inverted gate
	ELSE i_pulse WHEN 	m_output_enable = '0' AND m_monostable ='0' AND m_gate_enable ='1' AND m_gate_invert ='1' AND gate ='0'	-- astable with  inverted gate
	ELSE '0'     WHEN 	m_output_enable = '0' AND m_monostable ='0' AND m_gate_enable ='1' AND m_gate_invert ='1' AND gate ='1'	-- astable with  non inverted gate
	ELSE i_pulse WHEN 	m_output_enable = '0' AND m_monostable ='1' 												-- monostable
	ELSE '0';

	real_pulse <= pulse  WHEN m_output_invert ='0'
		ELSE NOT( pulse) WHEN m_output_invert ='1';

test(0) <= m_linked;
test(1) <= i_enable_skew;
test(2) <= done_skew;
test(3) <= m_monostable;
test(4) <= done_jitter;
--test(5) <= trigger_rising;
test(5) <= linked_monostable;
test(6) <= linked_burst_mono;

END rtl;





















