-- contains the pulse generator logic


LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY pulse_gen IS

	PORT
	(	
		clk	  				: IN  STD_LOGIC;
		reset				: IN  STD_LOGIC;
		control_wr_complete	: IN  STD_LOGIC;
		controlReg1 		: IN  STD_LOGIC_VECTOR(127 DOWNTO 0);
		controlReg2 		: IN  STD_LOGIC_VECTOR(127 DOWNTO 0);
		controlReg3 		: IN  STD_LOGIC_VECTOR(127 DOWNTO 0);
		updating_now		: IN  STD_LOGIC ;
		version				: IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
		statusReg  			: OUT STD_LOGIC_VECTOR(16 DOWNTO 0);
		pulse_1				: OUT STD_LOGIC;		
		pulse_2				: OUT STD_LOGIC;		
		gate 				: IN STD_LOGIC;	
		test				: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);

END pulse_gen;

ARCHITECTURE rtl OF pulse_gen IS
------------------------
-- Declare COMPONENTS --
------------------------
COMPONENT pulse IS
	PORT
	(	
		clk	  				: IN  STD_LOGIC;
		reset				: IN  STD_LOGIC;
		chan_num			: IN  STD_LOGIC;
		gate 				: IN  STD_LOGIC;	
		jitter				: IN  STD_LOGIC_VECTOR (15 DOWNTO 0);
	 	done_skew			: IN  STD_LOGIC;
		sync_in				: IN  STD_LOGIC;	
		control_wr_complete	: IN  STD_LOGIC;
		jitter_clk_enable	: IN  STD_LOGIC;
 		mode				: IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
		burst				: IN  STD_LOGIC_VECTOR( 7 DOWNTO 0);
		on_count			: IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
		off_count			: IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
	 	linked				: OUT  STD_LOGIC;
	 	enable_skew			: OUT STD_LOGIC;
		real_pulse			: OUT STD_LOGIC;		
		sync_out			: OUT STD_LOGIC;
		test				: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)

	);

END COMPONENT;

COMPONENT downcounter IS
	GENERIC
	(
		size			: NATURAL := 7 
	);
	PORT
	(	
		clk	 	 		: IN  STD_LOGIC;
		clken	  		: IN  STD_LOGIC;
		reset	  		: IN  STD_LOGIC;
		enable			: IN  STD_LOGIC;
		count			: IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
		done			: OUT STD_LOGIC
	);

END COMPONENT;

----------------------
-- Declare Signals  --
----------------------
 	SIGNAL mode_1			: STD_LOGIC_VECTOR ( 15 DOWNTO 0);
 	SIGNAL mode_2			: STD_LOGIC_VECTOR ( 15 DOWNTO 0);
	SIGNAL on_1				: STD_LOGIC_VECTOR (31 DOWNTO 0);
	SIGNAL on_2				: STD_LOGIC_VECTOR (31 DOWNTO 0);
	SIGNAL jitter_1			: STD_LOGIC_VECTOR (15 DOWNTO 0);
	SIGNAL jitter_2			: STD_LOGIC_VECTOR (15 DOWNTO 0);
	SIGNAL burst_1			: STD_LOGIC_VECTOR ( 7 DOWNTO 0);
	SIGNAL burst_2			: STD_LOGIC_VECTOR ( 7 DOWNTO 0);
	SIGNAL off_1			: STD_LOGIC_VECTOR (31 DOWNTO 0);
	SIGNAL off_2			: STD_LOGIC_VECTOR (31 DOWNTO 0);
	SIGNAL skew				: STD_LOGIC_VECTOR (31 DOWNTO 0);
	SIGNAL enable_skew		: STD_LOGIC;
	SIGNAL done_skew		: STD_LOGIC;
	SIGNAL sync_1_out		: STD_LOGIC;
	SIGNAL sync_2_out		: STD_LOGIC;
	SIGNAL pulse1test		: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL pulse2test		: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL jitter_clk_enable	: STD_LOGIC;
	SIGNAL random_jitter		: STD_LOGIC_VECTOR (15 DOWNTO 0);
	SIGNAL i_gate_1			: STD_LOGIC;
	SIGNAL gate_2			: STD_LOGIC;
	SIGNAL linked			: STD_LOGIC;


---------------------
-- Code Starts Here --
----------------------

BEGIN

--------------------------------------------
-- instantiate control & status registers --
--------------------------------------------
	on_1 <= controlReg1( 31 DOWNTO 0 );
	off_1 <= controlReg1( 63 DOWNTO 32 );
	jitter_1 <= controlreg1 ( 79 DOWNTO 64 );
	burst_1 <= controlReg1( 103 DOWNTO 96 );

	on_2 <= controlReg2( 31 DOWNTO 0 );
	off_2 <= controlReg2( 63 DOWNTO 32 );
	jitter_2 <= controlreg2 ( 79 DOWNTO 64 );
	burst_2 <= controlReg2( 103 DOWNTO 96 );

-- hold off updating the mode register until all the other registers are updated.
l_mode_update : PROCESS(clk, reset,updating_now,controlReg1,controlReg2)
BEGIN
	IF ( reset ='1' ) THEN
		mode_1 <= x"0000";
		mode_2 <= x"0000";
	ELSIF ( rising_edge(clk) ) THEN
		IF ( updating_now = '0' ) THEN
			mode_1 <= controlReg1( 95 DOWNTO 80 );
			mode_2 <= controlReg2( 95 DOWNTO 80 );
		END IF;
	END IF;
END PROCESS;
			
	

-- skew will allow pulse 1 and pulse 2 to be offset against each other.

	skew <= controlreg3( 31 DOWNTO 0 );
	statusReg(15 DOWNTO 0)   <= version(15 DOWNTO 0);
	statusReg(16) <= gate;

-- special routing for the gate to allow for linked operation.



-- instantiate  pulse generators
l_pulse_1 : pulse
	PORT MAP
	(	
		clk	  	=> 		clk ,
		reset		=> 		reset,
		chan_num	=>		'0',
		gate 		=> 		gate ,
 		mode		=>  	mode_1,
 		linked		=>  	linked,
		enable_skew => 		OPEN,
		sync_in		=>		sync_2_out,	
		done_skew 	=>		'1',
		control_wr_complete =>		control_wr_complete,	
		on_count	=> 		on_1,
		jitter		=> 		jitter_1,
		jitter_clk_enable => jitter_clk_enable,
		burst		=> 		burst_1,
		off_count	=> 		off_1,
		real_pulse	=> 		pulse_1 ,
		sync_out	=> 		sync_1_out,
		test		=>		pulse1test
	);

l_pulse_2 : pulse
	PORT MAP
	(	
		clk	  		=> 		clk ,
		reset		=> 		reset,
		chan_num	=>		'1',
		gate 		=> 		gate_2 ,
 		mode		=>  	mode_2,
 		linked		=>  	OPEN,
		sync_in		=>		sync_1_out,	
		on_count	=> 		on_2,
		enable_skew => 		enable_skew,
		done_skew	=>		done_skew,
		control_wr_complete =>		control_wr_complete,	
		jitter_clk_enable => jitter_clk_enable,
		jitter		=> 		jitter_2,
		burst		=> 		burst_2,
		off_count	=> 		off_2,
		real_pulse	=> 		pulse_2 ,
		sync_out	=> 		sync_2_out,
		test		=>		pulse2test
	);

gate_2 <= gate WHEN linked ='0'
		else sync_1_out ;
	
------------------------------------
-- instantiate skew downcounter	  --
------------------------------------
l_skew : downcounter
	GENERIC MAP 
	(
		size => 32
	)
	PORT MAP
	(	
		clk		=> clk,
		clken	=> '1',
		reset	=> reset,
		enable	=> enable_skew,
		count	=> skew,
		done	=> done_skew
	);
-- random sequence generator	
l_random_jitter : PROCESS ( clk,reset)
BEGIN
	IF ( reset ='1' ) THEN
		random_jitter <=x"0000";
	ELSIF ( rising_edge(clk ) ) THEN
		random_jitter <= random_jitter( 14 DOWNTO 0 ) & ( random_jitter(15) XNOR random_jitter(14));
	END IF;
END PROCESS;
	
jitter_clk_enable <= random_jitter(15);	


test(0) <=		pulse2test(0);
test(1) <=		pulse2test(1);
test(2) <=		pulse2test(2);
test(3) <=		pulse2test(3);
test(4) <=		pulse2test(4);
test(5) <=		pulse2test(5);
test(6) <=		pulse2test(6);
test(7) <=		pulse2test(7);






END rtl;




















