-- Title : pulse generator top level
-- Author : AM Hunt
-- Date : 08/01/05
-- history : first cut

-- contains bus interface and connectin logic to pulse generator
-- modified to use RS232 port for communication with the USB device.

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY pulse_generator_top_level IS

  PORT
    (
	clkin 	  			: IN 	STD_LOGIC;	-- global clk
	slowClkOut			: OUT	STD_LOGIC;	-- to be 12MHz
	SlowSysClk			: IN 	STD_LOGIC;  -- to be 12MHz

-- USB interface
	cts					: IN  STD_LOGIC;		-- RS232 clear to send to USB connects to usb RTS
	rxd					: IN  STD_LOGIC;		-- RS232 receive data to USB connects to usb TXD
	rts					: OUT  STD_LOGIC;		-- RS232 ready to send to FPGA connects to usb CTS
	txd					: OUT  STD_LOGIC;		-- RS232 transmit from FPGA connects to usb RXD

	dtr					: IN  STD_LOGIC;		-- RS232 deata terminal ready - will use as a reset at some point

-- pulse outputs
	pulse_1				: OUT STD_LOGIC;		
	pulse_2				: OUT STD_LOGIC;		
-- enable 	
	pulse_1_en			: OUT STD_LOGIC;		
	pulse_2_en			: OUT STD_LOGIC;		
	gate_en				: OUT STD_LOGIC;		
-- gate/sync input
	gate 				: IN STD_LOGIC;		

--	LEDS
	leds	    		: OUT STD_LOGIC_VECTOR(7 DOWNTO 0);

-- test stuff
	testIn 				: IN STD_LOGIC;
	resetOut 			: OUT STD_LOGIC

	);
							
END pulse_generator_top_level;

ARCHITECTURE rtl OF pulse_generator_top_level IS
	CONSTANT version	   	: STD_LOGIC_VECTOR(15 DOWNTO 0) := X"010e";
	SIGNAL   clk		 	: STD_LOGIC;  -- same as clk in
	SIGNAL   baudClkx16En 	: STD_LOGIC;  -- tx fifo NOT full
	SIGNAL   baudClkEn		: STD_LOGIC;  -- rx fifo NOT empty
	SIGNAL   extReset	  	: STD_LOGIC;
	SIGNAL   deviceReset 	: STD_LOGIC;
	
	SIGNAL   rd		  		: STD_LOGIC;
	SIGNAL   wr		  		: STD_LOGIC;
	SIGNAL   dataIn			: STD_LOGIC_VECTOR(7 DOWNTO 0);  -- data into the USB peripheral
	SIGNAL   dataOut	   	: STD_LOGIC_VECTOR(7 DOWNTO 0);  -- data out of the usb peripheral
	SIGNAL   txRequest	 	: STD_LOGIC;
	SIGNAL   readReq	   	: STD_LOGIC;
	SIGNAL   rdComplete		: STD_LOGIC;
	SIGNAL   wrComplete		: STD_LOGIC;
	SIGNAL   controlReg1	: STD_LOGIC_VECTOR(127 DOWNTO 0);
	SIGNAL   controlReg2	: STD_LOGIC_VECTOR(127 DOWNTO 0);
	SIGNAL   controlReg3	: STD_LOGIC_VECTOR(127 DOWNTO 0);
	SIGNAL   statusReg	 	: STD_LOGIC_VECTOR(16 DOWNTO 0);
	SIGNAL   status_data 	: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL   rdEn			: STD_LOGIC;
	SIGNAL   ileds			: STD_LOGIC_VECTOR(3 DOWNTO 0);
	SIGNAL   ftdi_data_rdy 	: STD_LOGIC;
	SIGNAL   ftdidataIn	 	: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL   addrIn	 		: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	controlWr		: STD_LOGIC;
	SIGNAL	status_Rd		: STD_LOGIC;
	SIGNAL 	packet_ram_data_out	: STD_LOGIC_VECTOR(7 DOWNTO 0);
 	SIGNAL  control_address	: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	control_wr_complete	: STD_LOGIC;
	SIGNAL	updating_now	: STD_LOGIC ;
	SIGNAL	iCts			: STD_LOGIC;		-- RS232 clear to send to USB connects to usb RTS

	-- clks
	SIGNAL	FastSysClk		: STD_LOGIC;
	
-- test signals

	SIGNAL	packetDecoderTest	: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	addrDecoderTest	: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	pulseTest		: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	rxRs232test		: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	txRs232test		: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL  itxd		  	: STD_LOGIC;
	SIGNAL	ipulse_1		: STD_LOGIC;		-- 
	SIGNAL	ipulse_2		: STD_LOGIC;		-- 
	SIGNAL	pulseLed		: STD_LOGIC;		-- 
	SIGNAL  stretchRXD    	: STD_LOGIC;

  COMPONENT pulse_gen

	PORT
	(
		clk	  				: IN  STD_LOGIC;
		reset				: IN  STD_LOGIC;
		controlReg1 		: IN  STD_LOGIC_VECTOR(127 DOWNTO 0);
		controlReg2 		: IN  STD_LOGIC_VECTOR(127 DOWNTO 0);
		controlReg3 		: IN  STD_LOGIC_VECTOR(127 DOWNTO 0);
		updating_now		: IN STD_LOGIC ;
		statusReg  			: OUT STD_LOGIC_VECTOR(16 DOWNTO 0);
		control_wr_complete	: IN  STD_LOGIC;
		version				: IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
		pulse_1				: OUT STD_LOGIC;		
		pulse_2				: OUT STD_LOGIC;		
		gate 				: IN STD_LOGIC;	
		test				: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	  );

  END COMPONENT;

  COMPONENT clocks
PORT
(
	reset					: IN	STD_LOGIC;
	clkin					: IN	STD_LOGIC; -- will be 48MHz
	SlowSysClk				: IN	STD_LOGIC; -- will be 12MHz
	baudClkx16En			: OUT 	STD_LOGIC; -- is slowSysClk/4 - 3MHz
	baudClkEn				: OUT 	STD_LOGIC; -- is slowSysClk/64 - 187.5kHz
	fastSysClk				: OUT	STD_LOGIC; -- currently 48M - will become 96 if get pll going
	slowClkOut				: OUT	STD_LOGIC  -- to be 12MHz
);
 END COMPONENT;

  COMPONENT addrdecoder
	PORT
	(
		clk					: IN STD_LOGIC;
		reset				: IN STD_LOGIC;
		deviceReset			: IN STD_LOGIC;
		controlWr			: IN STD_LOGIC;
		dataIn				: IN STD_LOGIC_VECTOR( 7 DOWNTO 0); 
		addrIn				: IN STD_LOGIC_VECTOR( 7 DOWNTO 0); 
		statusReg			: IN STD_LOGIC_VECTOR( 16 DOWNTO 0);
		
		controlReg1 		: OUT  STD_LOGIC_VECTOR(127 DOWNTO 0);
		controlReg2 		: OUT  STD_LOGIC_VECTOR(127 DOWNTO 0);
		controlReg3 		: OUT  STD_LOGIC_VECTOR(127 DOWNTO 0);
		status_data			: OUT STD_LOGIC_VECTOR( 7 DOWNTO 0); 
		test				: OUT	STD_LOGIC_VECTOR( 7 DOWNTO 0);
		leds				: OUT STD_LOGIC_VECTOR( 3 DOWNTO 0)
	);
  END COMPONENT;

COMPONENT  packet_decoder 
	PORT
	(
		reset				: IN STD_LOGIC ;	
		ftdi_data			: IN STD_LOGIC_vector(7 downto 0) 	;	-- pc_data bus 	
		status_data			: IN STD_LOGIC_vector(7 downto 0) 	;	-- datat from status registers
		ftdi_data_rdy		: IN STD_LOGIC;				 	-- data available form ftdi chip
		sysClk				: IN STD_LOGIC ;
		tx_done				: IN STD_LOGIC ;	-- data sent to IO fifo
		ftdi_wr				: OUT STD_LOGIC ;	-- write data to IO fifo
		status_rd			: OUT STD_LOGIC ;
		control_wr			: OUT STD_LOGIC ;
		control_wr_complete	: OUT STD_LOGIC ;
		data_out			: OUT STD_LOGIC_vector(7 downto 0) 	;	-- pc_data bus 	
		packet_ram_data_out	: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		control_address		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		updating_now		: OUT STD_LOGIC ;
		test				: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
		);
  END COMPONENT;

COMPONENT rs232_rx
	PORT
	(
		reset				: IN STD_LOGIC ;	
		clk         		: IN STD_LOGIC ;
		rxd         		: IN STD_LOGIC ;
		baudClkx16			: IN STD_LOGIC ; -- RS232 Baud Rate clkx16 enable.
		RXDataBuffer		: OUT STD_LOGIC_VECTOR ( 7 DOWNTO 0);
		stretchRXD 			: OUT STD_LOGIC;
		RXDataRdy   		: OUT STD_LOGIC;
		test				: OUT STD_LOGIC_VECTOR ( 7 DOWNTO 0)
	);
END COMPONENT;

COMPONENT rs232_tx 

PORT
(
		clk					: IN	STD_LOGIC;
		baudClkEn			: IN 	STD_LOGIC;  -- RS232 Baud Rate clk enable.
		reset				: IN	STD_LOGIC;
-- TX SIGNALS
		txRequest			: IN	STD_LOGIC ;  -- internal request to write data to ftdi chip
		cts					: IN	STD_LOGIC  ; -- clear to send from USB device
		dataOut				: IN 	STD_LOGIC_VECTOR( 7 DOWNTO 0);
		wrComplete			: OUT	STD_LOGIC;	-- internal signal that write to ftdi chip is complete
		txd					: OUT	STD_LOGIC;	-- transmit bit stream to USB device
		test				: OUT	STD_LOGIC_VECTOR ( 7 DOWNTO 0)
);
END COMPONENT;

BEGIN

-- make all signals active high
-- synchronise inputs

txd <= itxd ;
iCts <= NOT cts;
process ( clk, dtr)
BEGIN

	IF ( rising_edge (fastSysClk)) THEN
		deviceReset	<= dtr; -- OR softReset;
		extReset <= dtr;
	END IF;
END PROCESS;

	
--   RESEToUT   <= softReset;
resetOut <= dtr;

  l_clocks : clocks
	PORT MAP
	(
		reset   		=> extReset,
		clkin   		=> clkin,
		SlowSysClk		=> SlowSysClk,
		baudClkEn		=> baudClkEn,
		baudClkx16En	=> baudClkx16En,
		fastSysClk		=> fastSysClk,
		slowClkOut		=> slowClkOut
	);


  l_decode : addrdecoder
	PORT MAP
	(
		clk		 		=> slowSysClk,
		reset	  		=> extReset,
		deviceReset	 	=> deviceReset,
		dataIn			=> packet_ram_data_out,
		status_data		=> status_data,
		statusReg		=> statusReg,
		addrIn			=> control_address,
		controlWr		=> controlWr,
		controlReg1 	=> controlReg1,
		controlReg2 	=> controlReg2,
		controlReg3 	=> controlReg3,
		test			=> addrDecoderTest,
		leds			=> ileds
	);

  l_pulse_gen : pulse_gen

	PORT MAP
	(
		clk				=> fastSysClk,
		reset	   		=> deviceReset,
		controlReg1		=> controlReg1,
		controlReg2		=> controlReg2,
		controlReg3		=> controlReg3,
		statusReg		=> statusReg,
		control_wr_complete	=> control_wr_complete,
		version	 		=> version,
		updating_now	=> updating_now,
		pulse_1			=> ipulse_1,
		pulse_2			=> ipulse_2,
		gate			=> gate,
		test			=> pulseTest
	);
	
l_packet: packet_decoder
	PORT MAP
	(
		sysClk	 		=> slowSysClk,
		reset	 		=> deviceReset,
		ftdi_data		=> dataIn,
		ftdi_data_rdy	=> ftdi_data_rdy,
		status_data		=> status_data,
		tx_done			=> wrComplete,
		control_wr_complete	=> control_wr_complete,
		updating_now	=> updating_now,
		ftdi_wr			=> txRequest,
		status_rd		=> status_rd,
		control_wr		=> controlWr,
		data_out		=> dataOut,
		packet_ram_data_out 	=> packet_ram_data_out,
		control_address		=> control_address,
		test			=> packetDecoderTest
	);
	
 pulse_1 <= ipulse_1;
 pulse_2 <= ipulse_2;
pulseLed <= iPulse_1 AND iPulse_2;
	
l_rs232_rx: rs232_rx
	PORT MAP
	(
		reset   	=> deviceReset,
		clk	 		=> slowSysClk,
		rxd         => rxd,
		baudClkx16  => baudClkx16En,
-- TX SIGNALS
		RXDataBuffer=> dataIn,			-- received data
		stretchRXD  =>stretchRXD,
		RXDataRdy   => ftdi_data_rdy,  -- its ready
		test  		=> rxRs232test
	);

l_rs232_tx:rs232_tx 
	PORT MAP
	(
		clk	 		=> slowSysClk,
		reset   	=> deviceReset,
		baudClkEn  	=> baudClkEn,
-- TX SIGNALS
		txRequest  	=> txRequest,
	dataOut		=> dataOut,
--		cts			=> cts,
		wrComplete 	=> wrComplete,
--		cts			=> iCts,
		cts			=> '1',
		txd			=> itxd,
		test 		=>	txRs232test
);

	pulse_1_en		<= '0';
	pulse_2_en		<= '0';
	gate_en			<= '0';		




--	leds <= pulseTest(6 downto 0);
--	leds<= ileds; 
--  leds(0) <= packetDecoderTest(0);
leds(0) <= NOT gate;
leds(1)	<= NOT pulseLed;
leds(2)	<= NOT stretchRxd;


--  leds(1) <= baudClkEn;
--  leds(2) <= ftdi_data_rdy;
--  leds(3) <= baudClkx16En;
--  leds(4) <= rxd;
--  leds(5) <= packetDecoderTest(5);
-- leds( 3 downto 0 ) <= packetDecoderTest( 3 downto 0 );
--  leds(0) <= rxRs232test(0);
--  leds(1) <= rxRs232test(1);
--  leds(2) <= rxRs232test(2);
--  leds(4) <= updating_now;
--  leds(5) <= itxd;
--  leds(6) <= rxd;
--  leds(7) <= ipulse_2;
--  leds(4) <= packetDecoderTest(0);
--  leds(5) <= packetDecoderTest(1);
--  leds(6) <= packetDecoderTest(2);
--  leds(7) <= packetDecoderTest(3);



--  leds(7) <= rxRs232test(1);
--leds(1) <= packetDecoderTest(1);
--leds(2) <= ileds(2);
--leds(3) <= rxNotEmpty;

--leds(0) <= ileds(0);
--leds(1) <= ileds(1);
--leds(2) <= ileds(2);
--leds(3) <= ileds(3);

END rtl;












