-- Filename          : 
-- Modelname         : TB_PULSE_GENERATOR
-- Title             :
-- Purpose           :
-- Author(s)         : root
-- Comment           :
-- Assumptions       :
-- Limitations       :
-- Known errors      :
-- Specification ref :
-- ------------------------------------------------------------------------
-- Modification history:
-- ------------------------------------------------------------------------
-- Version  | Author | Date       | Changes made
-- ------------------------------------------------------------------------
-- 1.0      | root | 09.11.2003 | inital version


LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE std.textio.ALL;
USE IEEE.std_logic_textio.ALL;
ENTITY TB_PULSE_GENERATOR IS
END TB_PULSE_GENERATOR;

ARCHITECTURE BEH OF TB_PULSE_GENERATOR IS

  COMPONENT  pulse_generator_top_level IS

  PORT
    (
	clkin 	  		: IN STD_LOGIC;			-- global clk
	reset_n 		: IN STD_LOGIC;

-- USB interface
	txe_n 			: IN  STD_LOGIC;		-- tx fifo full
	rxf_n 			: IN  STD_LOGIC;		-- rx fifo empty ( names seem wrong )
	rd_n  			: OUT STD_LOGIC;		-- read data into FPGA
	extWr 			: OUT STD_LOGIC;		-- write data out of FPGA

	extUsbBus 		: INOUT STD_LOGIC_VECTOR(7 DOWNTO 0);

-- pulse outputs
	pulse_1			: OUT STD_LOGIC;		
	pulse_2			: OUT STD_LOGIC;		

-- gate/sync input
	gate 			: IN STD_LOGIC;		

--	LEDS
	leds	    		: OUT STD_LOGIC_VECTOR(3 DOWNTO 0);

-- test stuff
	resetOut 		: OUT STD_LOGIC

	);
END COMPONENT;

  CONSTANT PERIOD       : TIME := 10 ns;
  CONSTANT SAMPLEPERIOD : TIME := 50 ns;
  CONSTANT STOPTIME : TIME := 50 us;

  SIGNAL W_CLKIN        	: STD_LOGIC                     := '1';
  SIGNAL W_RESET_N      	: STD_LOGIC                     := '0';
  SIGNAL W_TXE_N        	: STD_LOGIC  ;
  SIGNAL W_RXF_N        	: STD_LOGIC                     := '1';
  SIGNAL W_RD_N 	        : STD_LOGIC;
  SIGNAL W_EXTWR       	 	: STD_LOGIC;
  SIGNAL W_EXTUSBBUS    	: STD_LOGIC_VECTOR (7 DOWNTO 0) := (OTHERS => 'Z');
  SIGNAL W_SAMPLECLK  		: STD_LOGIC                   :='1'  ;
  SIGNAL W_LEDS    	     	: STD_LOGIC_VECTOR (3 DOWNTO 0);
  SIGNAL W_RESETOUT		: STD_LOGIC;
  SIGNAL W_PULSE_1		: STD_LOGIC;		
  SIGNAL W_PULSE_2		: STD_LOGIC;		
  SIGNAL W_GATE 		: STD_LOGIC			:= '0';		
  SIGNAL endInit       		: STD_LOGIC                     := '0';
  SIGNAL startTest       	: STD_LOGIC                     := '0';
  SIGNAL stopTest        	: STD_LOGIC                     := '0';
  SIGNAL initVectorCnt   	: INTEGER                       := 1;
  SIGNAL runVectorCnt   	: INTEGER                       := 1;
  SIGNAL runDone         	: STD_LOGIC                     := '0';
  SIGNAL init_RXF_N      	: STD_LOGIC                     := '1';
  SIGNAL run_RXF_N      	: STD_LOGIC                     := '1';
  SIGNAL init_txe_N      	: STD_LOGIC                     := '1';
  SIGNAL run_txe_N       	: STD_LOGIC                     := '1';
 SIGNAL csum   			: STD_LOGIC_VECTOR (7 DOWNTO 0):= x"00";
 SIGNAL byteCount   		: STD_LOGIC_VECTOR (7 DOWNTO 0):= x"00";

 
	TYPE testData IS RECORD
                     usbData0 : STD_LOGIC_VECTOR (7 DOWNTO 0);
                     usbData1 : STD_LOGIC_VECTOR (7 DOWNTO 0);
	END RECORD;
  
  TYPE testFrameArray IS ARRAY (POSITIVE RANGE <>) OF testData;
  TYPE testArray0 IS ARRAY (POSITIVE RANGE <>) OF testData;
  TYPE testArray1 IS ARRAY (POSITIVE RANGE <>) OF testData;

-- framing for packet
  CONSTANT framing : testFrameArray :=
    (
-- -- send frame start sequence
      (x"aa" ,x"aa"),(x"aa" ,x"aa"),(x"aa" ,x"aa"),(x"aa" ,x"aa"),
      (x"aa" ,x"aa"),(x"aa" ,x"aa"),(x"aa" ,x"aa"),(x"aa" ,x"aa")
	);

  CONSTANT initTestVectors : testArray0 :=
--              DATA0,DATA1,DATA2,DATA3
    (
-- send packet of data
	(x"ad",x"ad"),(x"02",x"00"),	    	-- CONTROL = 02 - SO UPDATE CONTROL REG
	(x"10",x"00"),(x"00",x"00"),		-- controlreg1 on time lsb first
	(x"10",x"00"),(x"00",x"00"),		-- controlreg1 off time
	(x"ff",x"03"),(x"00",x"04"),			-- burst + mode
	(x"0c",x"0d"),(x"0e",x"0f"),
	(x"10",x"00"),(x"00",x"00"),		-- controlreg2 on time
	(x"10",x"00"),(x"00",x"00"),		-- controlreg2 off time
	(x"00",x"00"),(x"00",x"1b"),		-- burst + mode
	(x"1c",x"1d"),(x"1e",x"1f"),
	(x"00",x"00"),(x"00",x"00"),		-- controlreg3	skew
	(x"04",x"00"),(x"04",x"00"),		-- controlreg3	jitter1 jitter2
	(x"28",x"29"),(x"2a",x"2b"),
	(x"2c",x"2d"),(x"2e",x"2f"),
	(x"30",x"31"),(x"32",x"33"),		-- LEDS
	(x"34",x"35"),(x"36",x"37"),
	(x"38",x"39"),(x"3a",x"3b")
	);        
  CONSTANT stateMachine0 : testArray0 :=
--              DATA0,DATA1,DATA2,DATA3
    (
-- send packet of data
	(x"ad",x"ad"),(x"08",x"00"),	
	(x"00",x"01"),(x"02",x"03"),	
	(x"04",x"05"),(x"06",x"07"),
	(x"08",x"09"),(x"0a",x"0b"),
	(x"0c",x"0d"),(x"0e",x"0f"),
	(x"10",x"11"),(x"12",x"13"),	
	(x"14",x"15"),(x"16",x"17"),
	(x"18",x"19"),(x"1a",x"1b"),
	(x"1c",x"1d"),(x"1e",x"1f"),
	(x"20",x"21"),(x"22",x"23"),	
	(x"24",x"25"),(x"26",x"27"),
	(x"28",x"29"),(x"2a",x"2b"),
	(x"2c",x"2d"),(x"2e",x"2f"),
	(x"30",x"31"),(x"32",x"33"),	
	(x"34",x"35"),(x"36",x"37"),
	(x"38",x"39"),(x"3a",x"3b")
	);        

BEGIN

-- instanstiate analyser DUT
  DUT : pulse_generator_top_level
    PORT MAP
    (
      clkin           		=> W_CLKIN,
      reset_n         		=> W_RESET_N,
      txe_n           		=> '0' ,--W_TXE_N,
      rxf_n           		=> W_RXF_N,
      rd_n            		=> W_RD_N,
      extwr           		=> W_EXTWR,
      extusbbus    		=> W_EXTUSBBUS,
      pulse_1			=> W_PULSE_1,
      pulse_2			=> W_PULSE_2,
      gate			=> W_GATE,
      leds           		=> W_LEDS,
      resetout  	      	=> W_RESETOUT
      );

-- connect up clocks and other signals

  W_CLKIN <= '0' WHEN stopTest = '1'
             ELSE NOT W_CLKIN AFTER PERIOD/2;

  w_reset_n <= '1' AFTER 200 ns;
  startTest <= '1' AFTER 250 ns;


 W_GATE <= '1' AFTER 30 us; 
W_GATE <= '0' AFTER 45 us; 

-- record results to log file
  
l_write_data: PROCESS
 -- file cmdfile: TEXT;       -- Define the file 'handle'
 file results_file: text;
 variable line_in,line_out: line; -- Line buffers
 variable good: boolean;   -- Status of the read operations
 BEGIN
  	WAIT UNTIL startTest = '1';
	file_open(results_file,"results.txt",WRITE_MODE);
	ASSERT FALSE
                report "Text FILE OPEN"
                severity ERROR;
		write(line_out,"START");
		writeline(results_file,line_out);   

	WHILE ( stopTest ='0' ) LOOP
		WAIT UNTIL W_EXTWR ='1';  
		WAIT FOR 50 ns;
--			ASSERT FALSE
--				report "DATA"
--				severity ERROR;
			hwrite(line_out,W_EXTUSBBUS);
			writeline(results_file,line_out);   
			WAIT UNTIL W_EXTWR ='0';
	END LOOP;
-- 	wait until stopTest ='1';
	file_close( results_file);
	ASSERT FALSE
                report "Text FILE CLOSED"
                severity ERROR;
	WAIT;
    
  END PROCESS;	

-- initialise  
  l_init : PROCESS
    VARIABLE vector : testData;
  BEGIN
    WAIT UNTIL startTest = '1';
    ASSERT FALSE
	report "sending control data framing"
	severity ERROR;
    csum <=x"00";
    byteCount <= x"00";
    -- W_TXE_N <='0';
-- ********************************** send framing for control data***************************************
    FOR index IN framing'range LOOP
      --initVectorCnt           <= index;
      vector                  := framing(index);
      W_EXTUSBBUS(7 DOWNTO 0) <= vector.usbData0;
      init_rxf_n <= '0';
      WAIT UNTIL W_RD_N = '0';
      WAIT FOR PERIOD;
      init_rxf_n <= '1';
      WAIT UNTIL W_RD_N = '1';
      WAIT FOR PERIOD;
	W_EXTUSBBUS(7 DOWNTO 0) <= vector.usbData1;
 
      init_rxf_n <= '0';
      WAIT UNTIL W_RD_N = '0';
      WAIT FOR PERIOD;


      init_rxf_n <= '1';
      WAIT UNTIL W_RD_N = '1';
      WAIT FOR PERIOD;

    END LOOP;
    W_EXTUSBBUS <= (OTHERS => 'Z');

-- ********************************** send data ***************************************
    ASSERT FALSE
	report "sending control data "
	severity ERROR;

    FOR index IN initTestVectors'range LOOP
      --initVectorCnt           <= index;
      vector                  := initTestVectors(index);

       IF ( byteCount >x"3d") THEN
      		W_EXTUSBBUS(7 DOWNTO 0) <= csum ;
	ELSE
		W_EXTUSBBUS(7 DOWNTO 0) <= vector.usbData0;
		csum <= csum + vector.usbData0;
      END IF;
      
      init_rxf_n <= '0';
      WAIT UNTIL W_RD_N = '0';
      WAIT FOR PERIOD;

      init_rxf_n <= '1';
      WAIT UNTIL W_RD_N = '1';
      WAIT FOR PERIOD;
      byteCount <= byteCount + 1;

      IF ( byteCount >x"3d") THEN
      		W_EXTUSBBUS(7 DOWNTO 0) <= csum ;
	ELSE
		W_EXTUSBBUS(7 DOWNTO 0) <= vector.usbData1;
		csum <= csum + vector.usbData1;
      END IF;

      init_rxf_n <= '0';
      WAIT UNTIL W_RD_N = '0';
      WAIT FOR PERIOD;


      init_rxf_n <= '1';
      WAIT UNTIL W_RD_N = '1';
      WAIT FOR PERIOD;
      byteCount <= byteCount + 1;
    END LOOP;
    W_EXTUSBBUS <= (OTHERS => 'Z');
        ASSERT FALSE
	report " done init"
	severity ERROR;

    WAIT;
  END PROCESS;


l_readAfterRun : PROCESS
    VARIABLE vector    : testData;
    VARIABLE vectorCnt : INTEGER := 1;
  BEGIN
	WAIT FOR STOPTIME;
    stopTest <= '1';
    WAIT FOR PERIOD;

    WAIT;
 
  END PROCESS;

 w_rxf_n <= run_rxf_n AND init_rxf_n;
 w_txe_n <= run_txe_n AND init_txe_n;
 

END BEH;

CONFIGURATION CFG_TB_PULSE_GENERATOR OF TB_PULSE_GENERATOR IS
  FOR BEH
  END FOR;
END CFG_TB_PULSE_GENERATOR;




















